/* This file is part of Legendes II */
/* Copyright (c) 1998 by
 *   Philippe Clavel (philippe.clavel@supelec.fr)
 *   David Madore (david.madore@ens.fr) */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* File lgdlib.h */
/* This file contains the prototypes and declarations for the Legendes
 * basic portable library, contained in lgdlib.c */

/* Avoid multiple inclusions. */
#ifndef _LGDLIB_H
#define _LGDLIB_H

#include <stdio.h>
#include <sys/types.h>

#include "lgd0.h"
#include "config.h"

/* Just in case the system header files don't do a proper job. */
#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

/* Library types. */

#ifdef HAVE_LOWLEVEL_IO
typedef int lgd_filedesc_t;
#else
typedef FILE *lgd_filedesc_t;
#endif

/* File open modes. See comment on lgd_open() for a description. */

typedef enum {
  OPEN_READONLY,
  OPEN_READWRITE,
  OPEN_TRUNCATE
} lgd_open_mode_t;

/* A few error messages. */

#define MSG_OOM "Out of memory!"
#define MSG_CANT_OPEN_DEV_RANDOM "Can't open " DEV_RANDOM_PATH
#define MSG_CANT_READ_DEV_RANDOM "Can't read " DEV_RANDOM_PATH
#define MSG_CANT_OPEN_FILE "Can't open %s"
#define MSG_CANT_CLOSE_FILE "Can't close file"
#define MSG_CANT_SEEK_FILE "Can't seek file"
#define MSG_CANT_READ_FILE "Can't read file"
#define MSG_CANT_WRITE_FILE "Can't write file"

/* Now here are the prototypes. */

void lgd_fatal(char *fmt,...);

void lgd_free(void *ptr);

void *lgd_malloc(char die_on_failure,size_t size);

void *lgd_realloc(char die_on_failure,void *ptr,size_t size);

void lgd_randomize(void);

int lgd_random(int n);

int lgd_open(char die_on_failure,lgd_filedesc_t *f,char *name,
	     lgd_open_mode_t open_mode);

int lgd_close(char die_on_failure,lgd_filedesc_t f);

int lgd_seek(char die_on_failure,lgd_filedesc_t f,off_t offset,int whence);

off_t lgd_tell(char die_on_failure,lgd_filedesc_t f);

ssize_t lgd_size(char die_on_failure,lgd_filedesc_t f);

ssize_t lgd_read(char die_on_failure,lgd_filedesc_t f,void *buf,size_t count);

ssize_t lgd_write(char die_on_failure,lgd_filedesc_t f,void *buf,size_t count);

ssize_t lgd_read_littleendian_int32(char die_on_failure,lgd_filedesc_t f,
				    u_int32_t *buf,off_t off,size_t count);

ssize_t lgd_write_littleendian_int32(char die_on_failure,lgd_filedesc_t f,
				     u_int32_t *buf,off_t off,size_t count);

#endif /* not _LGDLIB_H */
