/* This file is part of Legendes II */
/* Copyright (c) 1998 by
 *   Philippe Clavel (philippe.clavel@supelec.fr)
 *   David Madore (david.madore@ens.fr) */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* File output.c */
/* This file contains the output handling thread. */

#include <stdio.h>
#include <pthread.h>

#include <X11/Xlib.h>
#include <X11/Intrinsic.h>

#include "lgd0.h"
#include "fifo.h"
#include "requests.h"

#include "ViewArea.h"
#include "output.h"
#include "input.h"

pthread_t output_thread;

char output_inited;

#define WIDTH 32
#define HEIGHT 32

/* Display coordinates of the upper-left corner */
long int disp_x,disp_y;
/* disp_redisplay_stat is 1 when the output thread has work to do. */
char disp_redisplay_stat;
/* The following condition serves to wake up the ouput thread when it
 * receives work to do. */
pthread_cond_t disp_redisplay_cond = PTHREAD_COND_INITIALIZER;
/* The following mutex serves to protect the above variable and
 * condition. */
pthread_mutex_t disp_redisplay_mutex = PTHREAD_MUTEX_INITIALIZER;

GC gc;

void
demand_redisplay(void)
     /* Wake up the output thread and ask it to redisplay.  This is
      * used by some other thread (a server thread, presumably).
      * _Not_ by the output thread (of course). */
{
  pthread_mutex_lock(&disp_redisplay_mutex);
  disp_redisplay_stat=1;
  pthread_cond_broadcast(&disp_redisplay_cond);
  pthread_mutex_unlock(&disp_redisplay_mutex);
}

void
init_40_output(void)
{
  disp_redisplay_stat=1;
  gc=XCreateGC(dpy,disp_pm,0,NULL);
}

void
output_loop(void)
{
  int i,j;
  long int x,y;
  /* Wait for a redisplay demand */
  pthread_mutex_lock(&disp_redisplay_mutex);
  while (!disp_redisplay_stat)
    pthread_cond_wait(&disp_redisplay_cond,&disp_redisplay_mutex);
  disp_redisplay_stat=0;
  pthread_mutex_unlock(&disp_redisplay_mutex);
  /* Where is the hero? */
  reqq_whereis_hero(&x,&y);
  disp_x=x-16;
  disp_y=y-16;
  /* Now we fill in the array */
  for (i=0;i<WIDTH;i++) for (j=0;j<HEIGHT;j++) {
    u_int32_t v;
    char h;
    x=disp_x+i;
    y=disp_y+j;
    reqq_readmap(x,y,&v,&h);
    if (v==0)
      XSetForeground(dpy,gc,0);
    else if (v==1)
      XSetForeground(dpy,gc,65535);
    XFillRectangle(dpy,disp_pm,gc,i*ICONSIZE,j*ICONSIZE,ICONSIZE,ICONSIZE);
    if (h) {
      XSetForeground(dpy,gc,32768);
      XFillArc(dpy,disp_pm,gc,i*ICONSIZE,j*ICONSIZE,ICONSIZE,ICONSIZE,0,23040);
    }
  }
  demand_redraw();
}

void *
output_main(void *garbage)
{
  while (1)
    output_loop();
}
