/* This file is part of Legendes II */
/* Copyright (c) 1998 by
 *   Philippe Clavel (philippe.clavel@supelec.fr)
 *   David Madore (david.madore@ens.fr) */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* File server.c */
/* This file implements the server thread, which is really the main
 * part of the game.  It answers queries from the other (client)
 * threads. */

#include <stdio.h>
#include <pthread.h>

#include "lgd0.h"
#include "lgdlib.h"
#include "fifo.h"
#include "mapchunks.h"

#include "output.h"

/* The identifier of the server thread */
pthread_t server_thread;

typedef struct hero_s {
  long int x;
  long int y;
} hero_t;

hero_t hero;

void
init_10_server(void)
{
  init_10_20_map_engine();
  hero.x=5;
  hero.y=5;
}
     
void *
server_main(void *garbage)
     /* The server thread's main function */
{
  while (1) {
    msg_fifo_t *msg;
    msg_response_t *resp;
    msg = wait_control_msg();
    resp = msg->response;
    switch (msg->type) {
    case msg_noop:
      if (resp) resp->errcode=msg_err_ok;
      break;
    case msg_die:
      abort();
      break;
    case msg_readmap : {
      long int x,y;
      if (resp) {
	x = msg->param.readmap.x;
	y = msg->param.readmap.y;
	resp->ret.readmap.val = read_map_square(0,x,y);
	resp->ret.readmap.hero = (hero.x==x)&&(hero.y==y);
	resp->errcode = msg_err_ok;
      }
    } break;
    case msg_write_chunk : {
      long int x,y;
      u_int32_t v;
      x = msg->param.write_chunk.x;
      y = msg->param.write_chunk.y;
      v = msg->param.write_chunk.v;
      write_map_chunk(0,x,y,v);
      demand_redisplay();
    } break;
    case msg_write_square : {
      long int x,y;
      u_int32_t n,v;
      x = msg->param.write_chunk.x;
      y = msg->param.write_chunk.y;
      v = msg->param.write_chunk.v;
      n = read_map_chunk(0,x,y);
      write_chunk_square(n,x,y,v);
      demand_redisplay();
    } break;
    case msg_sync_map :
      sync_chunk_cache();
      sync_zone_cache();
      break;
    case msg_move_hero : {
      long int xf,yf;
      xf = hero.x + msg->param.move_hero.dx;
      yf = hero.y + msg->param.move_hero.dy;
      if (read_map_square(0,xf,yf)==1) {
	hero.x = xf;
	hero.y = yf;
	if (resp) resp->errcode = msg_err_ok;
	demand_redisplay();
      } else {
	if (resp) resp->errcode = msg_err_blocked;
      }
    } break;
    case msg_whereis_hero :
      if (resp) {
	resp->ret.whereis_hero.x = hero.x;
	resp->ret.whereis_hero.y = hero.y;
	resp->errcode = msg_err_ok;
      }
      break;
    default :
      if (resp) resp->errcode = msg_err_unknown;
      break;
    }
    if (resp) resp->ques_serial = msg->serial;
    if (msg->in_heap)
      lgd_free(msg);
    else msg->treated=1;
    if (resp) {
      pthread_mutex_lock(&resp->ready_mutex);
      resp->ready_stat=1;
      pthread_cond_broadcast(&resp->ready_cond);
      pthread_mutex_unlock(&resp->ready_mutex);
    }
  }
}
